@@ -1,7 +1,7 @@
 /*
  * 
  * Rrdrand : physical random number if R on Intel CPU ivy bridge or later
- * Author: Eiji Nakama<nakama@com-one.com>, Junji Nakano <nakanoj@ism.ac.jp>
+ * Author: Eiji NAKAMA <nakama@com-one.com>, Junji NAKANO <nakanoj@ism.ac.jp>
  * Copyright(C) 2014
  */
 
@@ -24,16 +24,34 @@
 static int poolsize=POOLSIZE;
 static int poolpoint=0;
 static int poolavail=0;
-static double poolbuf[POOLSIZE];
+static unsigned int poolbuf[POOLSIZE];
 
 __inline static void mycpuid(int op, int *eax, int *ebx, int *ecx, int *edx){
 #if defined(HAVE_X86_CPUID) && HAVE_X86_CPUID == 1
+#if defined(__i386__) && defined(__PIC__)
+        __asm__ __volatile__("xchgl %%ebx, %k1;" /* save pic register */
+                             "cpuid;"
+                             "xchgl %%ebx, %k1;"
+                             : "=a"  (*eax),
+                               "=&r" (*ebx),
+                               "=c"  (*ecx),
+                               "=d"  (*edx) : "0" (op));
+#elif defined(__x86_64__) && defined(__PIC__)
+        __asm__ __volatile__("xchgq %%rbx, %q1;" /* save pic register */
+                             "cpuid;"
+                             "xchgq %%rbx, %q1;"
+                             : "=a"  (*eax),
+                               "=&r" (*ebx),
+                               "=c"  (*ecx),
+                               "=d"  (*edx) : "0" (op));
+#else
         __asm__ __volatile__("cpuid"
                              : "=a" (*eax),
                                "=b" (*ebx),
                                "=c" (*ecx),
                                "=d" (*edx) : "0" (op));
 #endif
+#endif
 }
 
 __inline static int has_rdrand(void)
@@ -69,7 +87,9 @@ __inline static unsigned int rdrand(void)
 #if defined(HAVE_X86_CPUID) && HAVE_X86_CPUID == 1
     do{
       r=0;
-      __asm__ __volatile__ ( ".byte 0x0f,0xc7,0xf0;" "setc %0" : "=qm" (r), "=a" (v));
+      __asm__ __volatile__ ( ".byte 0x0f,0xc7,0xf0;"
+                             "setc %0;"
+                             : "=qm" (r), "=a" (v));
     }while(r==0);
 #else
     v=0;
@@ -88,12 +108,13 @@ double *user_unif_rand ()
       return(&rn);
     }
     if(poolavail == 0 ){
-      for ( i=0; i<POOLSIZE ; i++ ){ /* 1.0/2^32  [0,1)*/
-	poolbuf[i]=(double)rdrand() * 2.3283064365386963e-10;
+      for ( i=0; i<POOLSIZE ; i++ ){
+	poolbuf[i]=rdrand();
       }
       poolavail=poolsize;poolpoint=0;
     }
-    rn=poolbuf[poolpoint];
+                                     /* 1.0/2^32  [0,1)*/
+    rn=(double)poolbuf[poolpoint]  * 2.3283064365386963e-10;
     poolavail--;
     poolpoint++;
 
